#ifndef __INTERNSCENE_H__
#define __INTERNSCENE_H__

#include "exttypes.h"
#include "geommath.h"
#include "internsurface.h"
#include "internspline.h"


class CTransformation
{
	friend class CConv3DS;
	friend class CExport;
	
public:
	const int32 GetNumPosKeys() const { return m_PosArray.GetNumKeys(); }
	const Vec3 GetPosKey(const int32 index) const { return m_PosArray.GetKey(index); }
	const int32 GetNumRotKeys() const { return m_RotArray.GetNumKeys(); }
	const Quat GetRotKey(const int32 index) const { return m_RotArray.GetKey(index); }
	const int32 GetNumScaleKeys() const { return m_ScaleArray.GetNumKeys(); }
	const float GetScaleKey(const int32 index) const { return m_ScaleArray.GetKey(index); }
	
protected:
	CInternKey3List m_PosArray;
	CInternQuatList m_RotArray;
	CInternKey1List m_ScaleArray;
};


class CInternMesh
{
	friend class CConv3DS;

public:
	int32 GetNumSurfaces()	{ return m_iNumSurfs; }
	CInternSurface& GetSurface(int32 iNumber)	{ return m_pSurfaces[iNumber]; }
	const char * GetName()	const { return m_sMeshName; }

private:
	int32 m_iNumSurfs;
	CInternSurface * m_pSurfaces;

	char * m_sMeshName;
};

class CInternMeshInstance : public CTransformation
{
	friend class CConv3DS;

public:
	CInternMesh& GetMesh()	{ return *m_pMesh; }
	const char * GetName()	const { return m_sMeshInstName; }
	CInternMeshInstance() : m_pMesh(NULL) {}

private:
	CInternMesh * m_pMesh;
	char * m_sMeshInstName;
};

class CInternLight : public CTransformation
{
	friend class CConv3DS;
	friend class CExport;

public:
	const int32 GetNumColorKeys() const { return m_ColorArray.GetNumKeys(); }
	const Vec3 GetColorKey(const int32 index) const { return m_ColorArray.GetKey(index); }
	const int32 GetNumNearKeys() const { return m_NearArray.GetNumKeys(); }
	const float GetNearKey(const int32 index) const { return m_NearArray.GetKey(index); }
	const int32 GetNumFarKeys() const { return m_FarArray.GetNumKeys(); }
	const float GetFarKey(const int32 index) const { return m_FarArray.GetKey(index); }

	
	bool IsSpot()	{ return m_bIsSpot; }
	const char * GetName() const	{ return m_sLightName; }

public:	// spot specific API
	bool IsRectangular()	{ return m_bIsRectangular; }
	const int32 GetNumHotSpotKeys() const { return m_HotSpotArray.GetNumKeys(); }
	const float GetHotSpotKey(const int32 index) const { return m_HotSpotArray.GetKey(index); }
	const int32 GetNumFallOffKeys() const { return m_FallOffArray.GetNumKeys(); }
	const float GetFallOffKey(const int32 index) const { return m_FallOffArray.GetKey(index); }
	const int32 GetNumAspectKeys() const { return m_AspectArray.GetNumKeys(); }
	const float GetAspectKey(const int32 index) const { return m_AspectArray.GetKey(index); }

private:
	
	CInternKey1List m_NearArray, m_FarArray;
	CInternKey3List m_ColorArray;

	bool m_bIsSpot;
	bool m_bIsRectangular;		// square spot

	CInternKey1List m_HotSpotArray, m_FallOffArray;
	CInternKey1List m_AspectArray;
	
	char * m_sLightName;
};

class CInternCamera : public CTransformation
{
	friend class CConv3DS;
	friend class CExport;

public:
	const int32 GetNumFovKeys() const { return m_FovArray.GetNumKeys(); }
	const float GetFovKey(const int32 index) const { return m_FovArray.GetKey(index); }
	int32 GetRatioX() const { return m_iRatioX; }
	int32 GetRatioY() const { return m_iRatioY; }
	const char * GetName() const { return m_sCamName; }

private:
	CInternKey1List m_FovArray;	// Total field of view in radians (fov < 180 degrees)
	int32 m_iRatioX;	// default value is 4
	int32 m_iRatioY;	// default value is 3
	char * m_sCamName;
};

class CInternScene
{
public:
	int32 GetNumMaterials()	{ return m_iNumMats; }
	int32 GetNumMeshes() { return m_iNumMeshes; }
	int32 GetNumMeshInstances() { return m_iNumMeshInstances; }
	int32 GetNumLights() { return m_iNumLights; }
	int32 GetNumCameras() { return m_iNumCameras; }
	CInternMaterial& GetMaterial(int iNumber) { return m_pMaterials[iNumber]; }
	CInternMesh& GetMesh(int iNumber) { return m_pMeshes[iNumber]; }
	CInternMeshInstance& GetMeshInstance(int iNumber) { return m_pMeshInstances[iNumber]; }
	CInternLight& GetLight(int iNumber) { return m_pLights[iNumber]; }
	CInternCamera& GetCamera(int iNumber) { return m_pCameras[iNumber]; }
	const float GetAnimTime() const { return m_fAnimTime; } // seconds
	const char * GetName() const { return m_sSceneName; }

	
protected:
	int32 m_iNumMeshes;
	int32 m_iNumMeshInstances;
	int32 m_iNumLights;
	int32 m_iNumCameras;
	
	CInternMesh * m_pMeshes;
	CInternMeshInstance * m_pMeshInstances;
	CInternLight * m_pLights;
	CInternCamera * m_pCameras;

protected:
	int32 m_iNumMats;
	CInternMaterial * m_pMaterials;

protected:
	char * m_sSceneName;

protected:
	float m_fAnimTime;	// in seconds
};

#endif
