/************************************
 * 3ds C++ File Loader					*
 * Author: Andrea Ingegneri			*
 * modifications: Morten Mikkelsen  *
 * Date: 16/12/2001						*
 * Distribution: LGPL (www.gnu.org) *
 ************************************/


#include "c3ds.h"
#include "keyf.h"
#include "VECTOR.h"
#include "MATRIX.h"

class C3DMaterial
{
public:
	string m_sMatName;

	string m_sDiffMapName;
	string m_sSpecMapName;
	string m_sReflMapName;
	string m_sBumpMapName;
};

class CDescriptor
{
public:
	CDescriptor(string Name, int Number) : m_Name(Name), m_Number(Number) {}
	string m_Name;
	int m_Number;
};

class C3DFace
{
public:
	C3DFace(int a, int b, int c) : A(a), B(b), C(c) {}
	int A, B, C;
};

class C3DCamera
{
public:
	C3DCamera(VECTOR Position, VECTOR Target, float Bank, float Lens, string cam_name) : 
																		position(Position),
																		target(Target),
																		bank(Bank),
																		lens(Lens),
																		m_sCamName(cam_name) {}
	VECTOR	position;
	VECTOR	target;
	float	bank, lens;
	string m_sCamName;
};

class C3DLight
{
public:
	C3DLight() : near(0), far(0), roll(0), hotspot(0), falloff(0), aspect(1), is_spot(false), is_rectangular(false), r(0), g(0), b(0) {}
	
	VECTOR	position;
	float		r, g, b;		// color
	float		near, far;
	
	bool		is_spot;
	bool		is_rectangular;		// square spot
	VECTOR	target;
	float		roll, hotspot, falloff;	// angles
	float		aspect;
	
	string m_sLightName;
};

class C3DObject
{
public:
	C3DObject() {}
	~C3DObject();
	void	UseTransform();
	vector	<VECTOR> vertices;
	vector	<VECTOR> texturePoints;
	vector	<C3DFace> faces;
	vector	<CDescriptor> descriptors;
	vector	<unsigned int> smoothrefs;
	MATRIX	transform;
	float	translate[3];

	string m_sObjName;
};

class C3DModel : public C3dsFileLoader
{
public:
	enum	{NoObjects = -1};
	C3DModel() : C3dsFileLoader(), mObjectsCount(NoObjects), mStillOpen(false), frame_begin(0), frame_end(0) {}
	bool	NewObject();
	bool	EndObject();
	bool	IsObjectWriting() const {return mStillOpen;}
	bool	AddVertex(float x, float y, float z);
	bool	AddFace(int A, int B, int C);
	bool	AddTexturePoint(float u, float v);
	bool	AddTriMesh();
	bool	AddCamera(C3DCamera Camera);
	bool	AddLight(float x, float y, float z);
	bool	AddMaterial();
	bool	AddTrackInfo();
	bool	AddTrackCamera();
	bool	AddTrackOmni();
	bool	AddTrackSpot();
	bool	AddTrackCamTarget();
	bool	AddTrackSpotTarget();
	bool	AddTrackPos(const keyf3 &key);
	bool	AddTrackRot(const keyf4 &key);
	bool	AddTrackScale(const keyf3 &key);
	bool	AddTrackFov(const keyf1 &key);
	bool	AddTrackRoll(const keyf1 &key);
	bool	AddTrackHotSpot(const keyf1 &key);
	bool	AddTrackFallOff(const keyf1 &key);
	bool	AddFaceDescriptor(string Name, int Number);
	bool	AddFaceSmoothRef(unsigned int smoothRef);
	bool	SetMatrix(MATRIX Mtx);
	bool	SetTranslation(float x, float y, float z);
	int		GetObjNumber() const {return (mObjectsCount + 1);}
	C3DObject *GetObject(int ObjectNumber) const {return mObjects[ObjectNumber];}
	int		GetCamNumber() const {return mCameras.size(); }
	C3DCamera GetCamera(int n) const { return mCameras[n]; }
	int		GetLightNumber() const {return mLights.size(); }
	C3DLight GetLight(int n) const { return mLights[n]; }
	int		GetMatNumber() const {return mMaterials.size(); }
	C3DMaterial GetMaterial(int n) const { return mMaterials[n]; }

	int GetNumMeshAnims() const { return mMeshAnim.size(); }
	int GetNumCameraAnims() const { return mCamAnim.size(); }
	int GetNumOmniAnims() const { return mOmniAnim.size(); }
	int GetNumSpotAnims() const { return mSpotAnim.size(); }
	int GetNumCameraTargetAnims() const { return mCamTargetAnim.size(); }
	int GetNumSpotTargetAnims() const { return mSpotTargetAnim.size(); }
	const C3DTrackInfo * GetTrackInfo(const string NameID) const;
	const C3DTrackCamera * GetTrackCamera(const string NameID) const;
	const C3DTrackOmni * GetTrackOmni(const string NameID) const;
	const C3DTrackSpot * GetTrackSpot(const string NameID) const;
	const C3DTrackCamTarget * GetTrackCamTarget(const string NameID) const;
	const C3DTrackSpotTarget * GetTrackSpotTarget(const string NameID) const;
	const C3DTrackInfo * GetTrackInfo(const int index) const { return &mMeshAnim[index]; }
	const C3DTrackCamera * GetTrackCamera(const int index) const { return &mCamAnim[index]; }
	const C3DTrackOmni * GetTrackOmni(const int index) const { return &mOmniAnim[index]; }
	const C3DTrackSpot * GetTrackSpot(const int index) const { return &mSpotAnim[index]; }
	const C3DTrackCamTarget * GetTrackCamTarget(const int index) const { return &mCamTargetAnim[index]; }
	const C3DTrackSpotTarget * GetTrackSpotTarget(const int index) const { return &mSpotTargetAnim[index]; }

	const int GetFrameBegin() const { return frame_begin; }
	const int GetFrameEnd() const { return frame_end; }

	
private:
	int		mObjectsCount;	// Objects counter
	bool	mStillOpen;		// If true an object creating session was opened but still not closed
	vector	<C3DObject*> mObjects;
	vector	<C3DCamera> mCameras;
	vector	<C3DLight> mLights;
	vector	<C3DMaterial> mMaterials;
	
private:
	vector	<C3DTrackInfo> mMeshAnim;	// just transformation anim of meshes
	vector	<C3DTrackCamera> mCamAnim;
	vector	<C3DTrackOmni> mOmniAnim;
	vector	<C3DTrackSpot>	mSpotAnim;
	vector	<C3DTrackCamTarget> mCamTargetAnim;
	vector	<C3DTrackSpotTarget> mSpotTargetAnim;

protected:
	void		UserRGBF(float r, float g, float b);
	void		UserObjName(string Name);
	void		User3dVert(float x, float y, float z);
	void		User3dFace(unsigned short A, unsigned short B, unsigned short C, unsigned short Flags);
	void		UserTriMesh();
	void		UserCamera(float px, float py, float pz,
								   float tx, float ty, float tz,
								   float Bank, float Lens);
	void		UserLight(float x, float y, float z);
	void		UserSpotLight(float x, float y, float z, float HotSpot, float FallOff);
	void		UserSetSpotRectangular();
	void		UserSpotRoll(float roll);
	void		UserSpotAspect(float aspect);
	void		UserInnerRange(float inner);
	void		UserOuterRange(float outer);

	void		UserMaterial();
	void		UserFaceMaterial(string Name, int Number);
	void		UserFaceSmoothGroup(unsigned int smoothRef);
	void		UserMapVertex(float U, float V);
	void		UserTransformMatrix(const Transform3dsMatrix &Transform, const Translate3dsMatrix &Translate);
	void		UserMatName(const string Name);
	void		UserMapFile(const string FileName);

	void		UserFrames(long Start, long End);
	void		UserTrackInfo();
	void		UserTrackCamera();
	void		UserTrackCamTgt();
	void		UserTrackLight();
	void		UserTrackLightTgt();
	void		UserTrackSpotL();
	void		UserTrackObjectName(const string Name, int Key1, int Key2, int Key3);
	void		UserTrackInstName( const string trackInstName );
	void		UserObjNumber(int ObjNumber);
	void		UserPivotPoint(float PivX, float PivY, float PivZ);
	void		UserTrackPos(int Frame, float t, float c, float b, float et, float ef, float x, float y, float z);
	void		UserTrackRot(int Frame, float t, float c, float b, float et, float ef, float x, float y, float z, float w);
	void		UserTrackScale(int Frame, float t, float c, float b, float et, float ef, float ScaleX, float ScaleY, float ScaleZ);
	void		UserTrackFov(int Frame, float t, float c, float b, float et, float ef, float fov);
	void		UserTrackRoll(int Frame, float t, float c, float b, float et, float ef, float roll);
	void		UserTrackHotSpot(int Frame, float t, float c, float b, float et, float ef, float hotspot);
	void		UserTrackFallOff(int Frame, float t, float c, float b, float et, float ef, float falloff);

	void		UserChunkObj();
	void		UserEndOfFile();

private:
	int		frame_begin;
	int		frame_end;
	
private:	// tmp working variable
	string	recName;
};
